package com.fjhx.account.service.account.impl;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.fjhx.account.entity.account.po.AccountDeptRunningWater;
import com.fjhx.account.entity.account.po.AccountDeptRunningWaterDetail;
import com.fjhx.account.entity.account.po.AccountRunningWater;
import com.fjhx.account.entity.account.vo.AccountRunningWaterVo;
import com.fjhx.account.mapper.account.AccountDeptRunningWaterDetailMapper;
import com.fjhx.account.service.account.AccountDeptRunningWaterDetailService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.common.constant.SourceConstant;
import com.fjhx.account.service.account.AccountDeptRunningWaterService;
import com.fjhx.account.service.account.AccountRunningWaterService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.account.entity.account.vo.AccountDeptRunningWaterDetailVo;
import com.fjhx.account.entity.account.dto.AccountDeptRunningWaterDetailSelectDto;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.fjhx.account.entity.account.dto.AccountDeptRunningWaterDetailDto;
import cn.hutool.core.bean.BeanUtil;

import java.util.List;


/**
 * <p>
 * 资金流水分拆部门表 服务实现类
 * </p>
 *
 * @author
 * @since 2023-04-13
 */
@Service
public class AccountDeptRunningWaterDetailServiceImpl extends ServiceImpl<AccountDeptRunningWaterDetailMapper, AccountDeptRunningWaterDetail> implements AccountDeptRunningWaterDetailService {
    @Autowired
    private AccountRunningWaterService accountRunningWaterService;

    @Autowired
    private AccountDeptRunningWaterService accountDeptRunningWaterService;

    @Override
    public Page<AccountDeptRunningWaterDetailVo> getPage(AccountDeptRunningWaterDetailSelectDto dto) {
        IWrapper<AccountDeptRunningWaterDetail> wrapper = getWrapper();
        wrapper.orderByDesc("adrwd", AccountDeptRunningWaterDetail::getId);
        Page<AccountDeptRunningWaterDetailVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    /**
     * 资金流水分拆部门表明细
     */
    @Override
    public AccountRunningWaterVo detail(Long id) {
        //查询资金流水信息
        AccountRunningWater accountRunningWater = accountRunningWaterService.getById(id);
        AccountRunningWaterVo accountRunningWaterVo = BeanUtil.toBean(accountRunningWater, AccountRunningWaterVo.class);

        //查询资金流水-部门关联表的信息(一个流水对应一个部门)
        AccountDeptRunningWater accountDeptRunningWater = accountDeptRunningWaterService.getOne(Wrappers.<AccountDeptRunningWater>lambdaQuery()
                .eq(AccountDeptRunningWater::getRunningWaterId, accountRunningWater.getId()));

        //查询资金流水分拆部门表明细
        List<AccountDeptRunningWaterDetail> accountDeptRunningWaterDetailList = this.list(Wrappers.<AccountDeptRunningWaterDetail>lambdaQuery()
                .eq(AccountDeptRunningWaterDetail::getDeptRunningWaterId, accountDeptRunningWater.getId()));

        //插入资金流水分拆部门表明细信息
        accountRunningWaterVo.setAccountDeptRunningWaterDetailList(accountDeptRunningWaterDetailList);
        //插入资金流水-部门关联表ID
        accountRunningWaterVo.setAccountDeptRunningWaterId(accountDeptRunningWater.getId());
        return accountRunningWaterVo;
    }

    /**
     * 资金流水分拆部门表新增
     */
    @Override
    public void add(AccountRunningWaterVo accountRunningWaterVo) {
        //修改资金流水-账户表的的分拆类型为已分拆
        AccountDeptRunningWater accountDeptRunningWater = new AccountDeptRunningWater();
        accountDeptRunningWater.setId(accountRunningWaterVo.getAccountDeptRunningWaterId());
        accountDeptRunningWater.setType("20");
        accountDeptRunningWaterService.updateById(accountDeptRunningWater);

        //删除资金流水分拆部门表数据
        this.remove(Wrappers.<AccountDeptRunningWaterDetail>lambdaQuery()
                .eq(AccountDeptRunningWaterDetail::getDeptRunningWaterId,accountRunningWaterVo.getAccountDeptRunningWaterId()));

        //添加资金流水分拆部门表数据
        List<AccountDeptRunningWaterDetail> deptRunningWaterDetailList = accountRunningWaterVo.getAccountDeptRunningWaterDetailList();

        deptRunningWaterDetailList.forEach(deptRunningWaterDetail->deptRunningWaterDetail
                .setDeptRunningWaterId(accountRunningWaterVo.getAccountDeptRunningWaterId()));

        this.saveBatch(deptRunningWaterDetailList);

    }

    @Override
    public void edit(AccountDeptRunningWaterDetailDto accountDeptRunningWaterDetailDto) {
        this.updateById(accountDeptRunningWaterDetailDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

}
