package com.fjhx.mes.service.production.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.fjhx.item.entity.product.po.ProductInfo;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.mes.entity.production.po.ProductionPlan;
import com.fjhx.mes.entity.production.po.ProductionTask;
import com.fjhx.mes.mapper.production.ProductionTaskMapper;
import com.fjhx.mes.service.production.ProductionPlanService;
import com.fjhx.mes.service.production.ProductionTaskService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.mes.utils.code.CodeEnum;
import com.ruoyi.system.utils.UserUtil;
import org.apache.ibatis.annotations.Param;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.mes.entity.production.vo.ProductionTaskVo;
import com.fjhx.mes.entity.production.dto.ProductionTaskSelectDto;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.fjhx.mes.entity.production.dto.ProductionTaskDto;
import cn.hutool.core.bean.BeanUtil;
import org.springframework.transaction.annotation.Transactional;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * <p>
 * 生产任务 服务实现类
 * </p>
 *
 * @author 
 * @since 2023-03-30
 */
@Service
public class ProductionTaskServiceImpl extends ServiceImpl<ProductionTaskMapper, ProductionTask> implements ProductionTaskService {
    @Autowired
    ProductInfoService productInfoService;
    @Autowired
    ProductionPlanService productionPlanService;

    @Override
    public Page<ProductionTaskVo> getPage(ProductionTaskSelectDto dto) {
        IWrapper<ProductionTask> wrapper = getWrapper();
        wrapper.orderByDesc("pt", ProductionTask::getId);
        if(ObjectUtil.isNotEmpty(dto.getKeyword())) {
            wrapper.and(q->q.like("pt",ProductionTaskVo::getCode, dto.getKeyword())
                    .or().like("pt",ProductionTaskVo::getQuantity, dto.getKeyword())
                    .or().like("pt",ProductionTaskVo::getDueDate, dto.getKeyword()));
        }
        Page<ProductionTaskVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<ProductionTaskVo> records = page.getRecords();
        //赋值产品名称
        List<Long> productIds = records.stream().map(ProductionTaskVo::getProductId).collect(Collectors.toList());
        if(ObjectUtil.isNotEmpty(productIds)) {
            List<ProductInfo> productInfos = productInfoService.listByIds(productIds);
            Map<Long, ProductInfo> productInfoMap = productInfos.stream().collect(Collectors.groupingBy(ProductInfo::getId,
                    Collectors.collectingAndThen(Collectors.toList(), value -> value.get(0))));
            for (ProductionTaskVo productionTaskVo : records) {
                Long productId = productionTaskVo.getProductId();
                if (ObjectUtil.isNotEmpty(productId)) {
                    ProductInfo productInfo = productInfoMap.get(productId);
                    if (ObjectUtil.isNotEmpty(productInfo)) {
                        productionTaskVo.setProductName(productInfo.getName());
                    }
                }
            }
        }

        //赋值负责人名字
        UserUtil.assignmentNickName(records,ProductionTask::getPersonLiableId,ProductionTaskVo::setPersonLiableName);
        return page;
    }

    @Override
    public ProductionTaskVo detail(Long id) {
        ProductionTaskVo productionTaskVo = baseMapper.detail(id);
        ProductInfo productInfo = productInfoService.getById(productionTaskVo.getProductId());
        if(ObjectUtil.isNotEmpty(productInfo)) {
            productionTaskVo.setProductName(productInfo.getName());
        }
        Map<Long, String> nickNameMapByIds = UserUtil.getNickNameMapByIds(Arrays.asList(productionTaskVo.getPersonLiableId()));
        if(ObjectUtil.isNotEmpty(nickNameMapByIds)) {
            productionTaskVo.setPersonLiableName(nickNameMapByIds.get(productionTaskVo.getPersonLiableId()));
        }
        return productionTaskVo;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(ProductionTaskDto productionTaskDto) {
        productionTaskDto.setCode(CodeEnum.PRODUCTION_TASK.getCode());
        this.save(productionTaskDto);
        Long productionPlanId = productionTaskDto.getProductionPlanId();
        //更新生产计划状态为进行中
        ProductionPlan byId = productionPlanService.getById(productionPlanId);
        byId.setStatus(1);
        productionPlanService.updateById(byId);
    }

    @Override
    public void edit(ProductionTaskDto productionTaskDto) {
        this.updateById(productionTaskDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    @Override
    public List<ProductionTaskVo> myListByIds(List<Long> ids){
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.in("pt.id",ids);
        return baseMapper.myListByIds(queryWrapper);
    }

}
