package com.fjhx.purchase.service.arrival.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.common.constant.SourceConstant;
import com.fjhx.purchase.entity.arrival.dto.ArrivalDto;
import com.fjhx.purchase.entity.arrival.dto.ArrivalSelectDto;
import com.fjhx.purchase.entity.arrival.po.Arrival;
import com.fjhx.purchase.entity.arrival.po.ArrivalDetail;
import com.fjhx.purchase.entity.arrival.vo.ArrivalVo;
import com.fjhx.purchase.entity.purchase.po.Purchase;
import com.fjhx.purchase.mapper.arrival.ArrivalMapper;
import com.fjhx.purchase.service.WdlyService;
import com.fjhx.purchase.service.arrival.ArrivalDetailService;
import com.fjhx.purchase.service.arrival.ArrivalService;
import com.fjhx.purchase.service.purchase.PurchaseDetailService;
import com.fjhx.purchase.service.purchase.PurchaseService;
import com.fjhx.purchase.util.code.CodeEnum;
import com.fjhx.wms.entity.stock.po.StockWait;
import com.fjhx.wms.entity.stock.po.StockWaitDetails;
import com.fjhx.wms.service.stock.StockWaitDetailsService;
import com.fjhx.wms.service.stock.StockWaitService;
import com.ruoyi.common.utils.wrapper.IWrapper;
import io.seata.spring.annotation.GlobalTransactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;


/**
 * <p>
 * 到货 服务实现类
 * </p>
 *
 * @author
 * @since 2023-04-07
 */
@DS(SourceConstant.PURCHASE)
@Service
public class ArrivalServiceImpl extends ServiceImpl<ArrivalMapper, Arrival> implements ArrivalService {

    @Autowired
    private StockWaitService stockWaitService;

    @Autowired
    private ArrivalDetailService arrivalDetailService;

    @Autowired
    private PurchaseDetailService purchaseDetailService;

    @Autowired
    private PurchaseService purchaseService;
    @Autowired
    private StockWaitDetailsService stockWaitDetailsService;


    @Override
    public Page<ArrivalVo> getPage(ArrivalSelectDto dto) {
        IWrapper<Arrival> wrapper = getWrapper();
        wrapper.orderByDesc("a", Arrival::getId);
        Page<ArrivalVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @Override
    public ArrivalVo detail(Long id) {
        Arrival Arrival = this.getById(id);
        ArrivalVo result = BeanUtil.toBean(Arrival, ArrivalVo.class);
        return result;
    }

    /**
     * 添加
     *
     * @param arrival
     */
    @GlobalTransactional
    @Override
    public void add(Arrival arrival) {
        arrival.setCode(CodeEnum.ARRIVAL.getCode());
        this.save(arrival);
        //修改采购状态
        Purchase purchase = new Purchase();
        purchase.setArrivalStatus(arrival.getArrivalStatus());
        purchase.setId(arrival.getPurchaseId());
        purchaseService.updateById(purchase);
        //取出到货明细
        List<ArrivalDetail> arrivalDetailList = arrival.getArrivalDetailList();
        if (CollectionUtils.isNotEmpty(arrivalDetailList)) {
            List<StockWait> stockWaitList = new ArrayList<>();
            for (ArrivalDetail d : arrivalDetailList) {
                //todo 这一块是做了采购到货状态修改。前端已经完成
                //查询采购明细
//                PurchaseDetail purchaseDetail = purchaseDetailService.getById(d.getPurchaseDetailId());
                //查询当前到货总数--修改对应得申购状态
//                BigDecimal arrivalCount = arrivalDetailService.getSumCount(d.getPurchaseDetailId(),d.getBussinessId());
//                BigDecimal sumArrivalCount = arrivalCount.add(d.getCount());
//                if(sumArrivalCount.compareTo(purchaseDetail.getCount())<0){//判断到货数量是否大于等于采购数量
//
//
//                }
                d.setArrivalId(arrival.getId());
                StockWait stockWait = new StockWait();
                stockWait.setProductId(d.getBussinessId());
                stockWait.setBusinessId(d.getId());//到货明细ID
                stockWait.setType(1);//入库
                stockWait.setStatus(0);//待入库
                stockWait.setBusinessType(3);//采购到货
                stockWait.setQuantity(d.getCount());
                stockWaitList.add(stockWait);
            }
            arrivalDetailService.saveBatch(arrivalDetailList);
            stockWaitService.saveBatch(stockWaitList);
        }
    }

    /**
     * 维多利亚到货
     */
    @GlobalTransactional
    @Override
    public void addByWdly(Arrival arrival) {
        arrival.setCode(CodeEnum.ARRIVAL.getCode());
        this.save(arrival);
        //修改采购状态
        Purchase purchase = new Purchase();
        purchase.setArrivalStatus(arrival.getArrivalStatus());
        purchase.setId(arrival.getPurchaseId());
        purchaseService.updateById(purchase);
        //创建待入库记录
        String victoriatouristJson = arrival.getVictoriatouristJson();
        JSONObject json = JSONObject.parseObject(victoriatouristJson);
        //修改发货记录的状态为到货
        Long deliverGoodsId = json.getLong("deliverGoodsId");
        WdlyService wdly = SpringUtil.getBean(WdlyService.class);
        wdly.updateDeliverGoodsStatus(deliverGoodsId);
        //----------
        StockWait stockWait = new StockWait();
        stockWait.setType(1);//入库
        stockWait.setStatus(0);//待入库
        stockWait.setBusinessType(3);//采购到货
        //将业务id设置成到货id
        stockWait.setBusinessId(arrival.getId());
        stockWait.setVictoriatouristJson(victoriatouristJson);
        stockWaitService.save(stockWait);
        //取出到货明细
        List<ArrivalDetail> arrivalDetailList = arrival.getArrivalDetailList();
        if (CollectionUtils.isNotEmpty(arrivalDetailList)) {
            List<StockWaitDetails> stockWaitDetailsList = new ArrayList<>();
            for (ArrivalDetail d : arrivalDetailList) {
                d.setArrivalId(arrival.getId());
                StockWaitDetails stockWaitDetails = new StockWaitDetails();
                stockWaitDetails.setStockWaitId(stockWait.getId());
                //业务明细id设置成到货明细id
                stockWaitDetails.setBusinessDetailsId(d.getId());
                stockWaitDetails.setProductId(d.getBussinessId());
                stockWaitDetails.setQuantity(d.getCount());
                stockWaitDetailsList.add(stockWaitDetails);
            }
            arrivalDetailService.saveBatch(arrivalDetailList);
            stockWaitDetailsService.saveBatch(stockWaitDetailsList);
        }
    }

    @Override
    public void edit(ArrivalDto arrivalDto) {
        this.updateById(arrivalDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

}
