package com.fjhx.wms.service.stock.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.fjhx.item.entity.product.po.ProductInfo;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.wms.entity.stock.po.Stock;
import com.fjhx.wms.entity.stock.po.StockCheck;
import com.fjhx.wms.entity.stock.po.StockCheckDetails;
import com.fjhx.wms.entity.stock.vo.StockCheckDetailsVo;
import com.fjhx.wms.entity.stock.vo.StockJournalVo;
import com.fjhx.wms.entity.stock.vo.StockVo;
import com.fjhx.wms.mapper.stock.StockCheckMapper;
import com.fjhx.wms.service.stock.StockCheckDetailsService;
import com.fjhx.wms.service.stock.StockCheckService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.system.utils.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.wms.entity.stock.vo.StockCheckVo;
import com.fjhx.wms.entity.stock.dto.StockCheckSelectDto;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.fjhx.wms.entity.stock.dto.StockCheckDto;
import cn.hutool.core.bean.BeanUtil;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * <p>
 * 库存盘点 服务实现类
 * </p>
 *
 * @author
 * @since 2023-03-22
 */
@Service
public class StockCheckServiceImpl extends ServiceImpl<StockCheckMapper, StockCheck> implements StockCheckService {

    @Autowired
    StockCheckDetailsService stockCheckDetailsService;
    @Autowired
    ProductInfoService productInfoService;

    @Override
    public Page<StockCheckVo> getPage(StockCheckSelectDto dto) {
        IWrapper<StockCheck> wrapper = getWrapper();
        wrapper.like("sc",StockCheckVo::getCreateTime,dto.getKeyword());
        wrapper.orderByDesc("sc", StockCheck::getId);
        Page<StockCheckVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        //根据id赋值操作人名称
        UserUtil.assignmentNickName(page.getRecords(), StockCheckVo::getCreateUser, StockCheckVo::setUserName);
        return page;
    }

    @Override
    public StockCheckVo detail(Long id) {
        StockCheckVo stockCheckVo = baseMapper.detail(id);

        List<StockCheckDetailsVo> list = stockCheckVo.getList();

        List<Long> productIds = list.stream().map(StockCheckDetailsVo::getProductId).collect(Collectors.toList());
        if (ObjectUtil.isNotEmpty(productIds)) {
            List<ProductInfo> productInfos = productInfoService.listByIds(productIds);
            Map<Long, ProductInfo> productInfoMap = productInfos.stream().collect(Collectors.groupingBy(ProductInfo::getId,
                    Collectors.collectingAndThen(Collectors.toList(), value -> value.get(0))));
            for (StockCheckDetailsVo stockVo : list) {
                ProductInfo productInfo = productInfoMap.get(stockVo.getProductId());
                stockVo.setProductName(productInfo.getName());
            }
        }
        return stockCheckVo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void add(StockCheckDto stockCheckDto) {
        stockCheckDto.setId(IdWorker.getId());
        List<StockCheckDetails> list = stockCheckDto.getList();
        for (StockCheckDetails stockCheckDetails : list){
            stockCheckDetails.setStockCheckId(stockCheckDto.getId());
            stockCheckDto.setResult(0);
            //判断盘点数量与库存是否相同，不同将盘点结果设置为异常
            if(stockCheckDetails.getQuantity()!=stockCheckDetails.getCheckQuantity()){
                stockCheckDto.setResult(1);
                break;
            }
        }
        //保存盘点记录以及明细
        save(stockCheckDto);
        stockCheckDetailsService.saveBatch(list);
    }

    @Override
    public void edit(StockCheckDto stockCheckDto) {
        this.updateById(stockCheckDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

}
