package com.fjhx.wms.service.stock.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.wms.entity.stock.dto.StockDto;
import com.fjhx.wms.entity.stock.dto.StockJournalDetailsDto;
import com.fjhx.wms.entity.stock.dto.StockSelectDto;
import com.fjhx.wms.entity.stock.po.Stock;
import com.fjhx.wms.entity.stock.po.StockJournal;
import com.fjhx.wms.entity.stock.po.StockJournalDetails;
import com.fjhx.wms.entity.stock.vo.StockVo;
import com.fjhx.wms.entity.warehouse.po.Warehouse;
import com.fjhx.wms.mapper.stock.StockMapper;
import com.fjhx.wms.service.stock.StockJournalDetailsService;
import com.fjhx.wms.service.stock.StockJournalService;
import com.fjhx.wms.service.stock.StockService;
import com.fjhx.wms.service.warehouse.WarehouseService;
import com.obs.services.internal.ServiceException;
import com.ruoyi.common.utils.wrapper.IWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * <p>
 * 库存 服务实现类
 * </p>
 *
 * @author
 * @since 2023-03-20
 */
@Service
public class StockServiceImpl extends ServiceImpl<StockMapper, Stock> implements StockService {
    @Autowired
    StockJournalService stockJournalService;
    @Autowired
    StockJournalDetailsService stockJournalDetailsService;
    @Autowired
    ProductInfoService productInfoService;
    @Autowired
    WarehouseService warehouseService;

    @Override
    public Page<StockVo> getPage(StockSelectDto dto) {
        IWrapper<Stock> wrapper = getWrapper();
        wrapper.orderByDesc("s", Stock::getId);
        wrapper.eq(Stock::getWarehouseId, dto.getId());
        wrapper.like(Stock::getQuantity, dto.getKeyword());

        Page<StockVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<StockVo> stockVos = page.getRecords();
        //获取产品id列表
        List<Long> productIds = stockVos.stream().map(Stock::getProductId).collect(Collectors.toList());
        if (ObjectUtil.isNotEmpty(productIds)) {
            //赋值产品名称
            productInfoService.attributeAssign(stockVos, StockVo::getProductId, (item, productInfo) -> {
                item.setType(productInfo.getType());
                item.setProductName(productInfo.getName());
            });
        }
        return page;
    }

    /**
     * 维多利亚 按仓库库存查询
     */
    @Override
    public Page<StockVo> pageByWarehouse(StockSelectDto dto) {
        Page<StockVo> page = getPage(dto);
        List<StockVo> stockVos = page.getRecords();

        //获取产品id列表
        List<Long> productIds = stockVos.stream().map(Stock::getProductId).collect(Collectors.toList());
        if (ObjectUtil.isNotEmpty(productIds)) {
            //赋值产品名称
            productInfoService.attributeAssign(stockVos, StockVo::getProductId, (item, productInfo) -> {
                //赋值维多利亚扩展产品信息
                String victoriatouristJson = item.getVictoriatouristJson();
                if (ObjectUtil.isNotEmpty(victoriatouristJson)) {
                    JSONObject json = JSONObject.parseObject(victoriatouristJson);
                    json.put("code", productInfo.getCode());
                    json.put("spec", productInfo.getSpec());
                    json.put("unit", productInfo.getUnit());
                    item.setVictoriatouristJson(json.toJSONString());
                }
            });
        }

        //维多利亚扩展
        List<Long> warehouseIds = new ArrayList<>();
        for (StockVo stockVo : stockVos) {
            String victoriatouristJson = stockVo.getVictoriatouristJson();
            if (ObjectUtil.isNotEmpty(victoriatouristJson)) {
                JSONObject json = JSONObject.parseObject(victoriatouristJson);
                //遍历仓库id
                warehouseIds.add(json.getLong("warehouseId"));
            }
        }
        //如果仓库id列表为空直接返回
        if (ObjectUtil.isEmpty(warehouseIds)) {
            return page;
        }
        List<Warehouse> warehouses = warehouseService.listByIds(warehouseIds);
        Map<Long, String> warehouseMap = warehouses.stream().collect(Collectors.toMap(Warehouse::getId, Warehouse::getName));
        for (StockVo stockVo : stockVos) {
            String victoriatouristJson = stockVo.getVictoriatouristJson();
            if (ObjectUtil.isNotEmpty(victoriatouristJson)) {
                JSONObject json = JSONObject.parseObject(victoriatouristJson);
                String warehouseName = warehouseMap.get(json.getLong("warehouseId"));
                json.put("warehouseName", warehouseName);
                stockVo.setVictoriatouristJson(json.toJSONString());
            }
        }
        return page;
    }

    @Override
    public Page<StockVo> pageByProduct(StockSelectDto dto) {
        IWrapper<Stock> wrapper = getWrapper();
        wrapper.orderByDesc("s", Stock::getId);
        wrapper.eq(Stock::getWarehouseId, dto.getId());
        wrapper.like(Stock::getQuantity, dto.getKeyword());

        Page<StockVo> page = this.baseMapper.pageByProduct(dto.getPage(), wrapper);
        List<StockVo> stockVos = page.getRecords();
        //获取产品id列表
        List<Long> productIds = stockVos.stream().map(Stock::getProductId).collect(Collectors.toList());
        if (ObjectUtil.isNotEmpty(productIds)) {
            //赋值产品名称
            productInfoService.attributeAssign(stockVos, StockVo::getProductId, (item, productInfo) -> {
                item.setType(productInfo.getType());
                item.setProductName(productInfo.getName());
                //赋值维多利亚扩展产品信息
                String victoriatouristJson = item.getVictoriatouristJson();
                if (ObjectUtil.isNotEmpty(victoriatouristJson)) {
                    JSONObject json = JSONObject.parseObject(victoriatouristJson);
                    json.put("code", productInfo.getCode());
                    json.put("spec", productInfo.getSpec());
                    json.put("unit", productInfo.getUnit());
                    item.setVictoriatouristJson(json.toJSONString());
                }
            });
        }
        return page;
    }

    @Override
    public Stock detail(StockDto dto) {
        return getOne(q -> q.eq(Stock::getWarehouseId, dto.getWarehouseId()).eq(Stock::getProductId, dto.getProductId()));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void add(StockDto stockDto) {
        //创建出入库记录
        StockJournal stockJournal = new StockJournal();
        stockJournal.setType(stockDto.getType());
        stockJournal.setWarehouseId(stockDto.getWarehouseId());
        stockJournal.setBusinessId(stockDto.getBusinessId());
        //保存出入库记录
        stockJournalService.save(stockJournal);
        //操作库存
        List<StockJournalDetails> stockJournalDetailsList = ModifyInventory(stockJournal.getId(), 1, stockDto.getList(), stockDto.getWarehouseId());
        //保存出入库明细
        stockJournalDetailsService.saveBatch(stockJournalDetailsList);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void edit(StockDto stockDto) {
        //创建出入库记录
        StockJournal stockJournal = new StockJournal();
        stockJournal.setType(stockDto.getType());
        stockJournal.setWarehouseId(stockDto.getWarehouseId());
        stockJournal.setBusinessId(stockDto.getBusinessId());
        //保存出入库记录
        stockJournalService.save(stockJournal);
        //操作库存
        List<StockJournalDetails> stockJournalDetailsList = ModifyInventory(stockJournal.getId(), 2, stockDto.getList(), stockDto.getWarehouseId());
        //保存出入库明细
        stockJournalDetailsService.saveBatch(stockJournalDetailsList);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void stockTransfer(StockDto stockDto) {
        //创建出入库记录
        StockJournal stockJournal = new StockJournal();
        stockJournal.setType(3);
        stockJournal.setWarehouseId(stockDto.getWarehouseId());
        stockJournal.setToWarehouseId(stockDto.getToWarehouseId());
        //保存出入库记录
        stockJournalService.save(stockJournal);
        //操作库存
        List<StockJournalDetails> stockJournalDetailsList = ModifyInventory(stockJournal.getId(), 2, stockDto.getList(), stockDto.getWarehouseId());
        ModifyInventory(stockJournal.getId(), 1, stockDto.getList(), stockDto.getToWarehouseId());
        //保存出入库明细
        stockJournalDetailsService.saveBatch(stockJournalDetailsList);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    /**
     * 操作库存通用方法
     *
     * @param stockJournalId 出入库记录id
     * @param type           1入库 2出库
     */
    @Override
    public List<StockJournalDetails> ModifyInventory(Long stockJournalId, int type, List<Stock> list, Long warehouseId) {
        List<Stock> data = new ArrayList<>();

        //获取现有库存
        List<Long> productIds = list.stream().map(Stock::getProductId).collect(Collectors.toList());
        List<Stock> stockList = list(q -> q.in(Stock::getProductId, productIds).eq(Stock::getWarehouseId, warehouseId));
        Map<Long, Stock> stockMap = stockList.stream().collect(Collectors.groupingBy(Stock::getProductId,
                Collectors.collectingAndThen(Collectors.toList(), value -> value.get(0))));

        List<StockJournalDetails> stockJournalDetailsList = new ArrayList<>();
        for (Stock stock : list) {
            Stock newStocks = new Stock();
            newStocks.setWarehouseId(warehouseId);
            newStocks.setProductId(stock.getProductId());
            newStocks.setQuantity(stock.getQuantity());

            //合并库存数量
            Stock oldStocks = stockMap.get(stock.getProductId());
            //用来存储操作数据
            if (ObjectUtil.isNotEmpty(oldStocks)) {
                BigDecimal quantity = oldStocks.getQuantity();
                if (type == 1) {
                    //入库库存相加
                    quantity = quantity.add(stock.getQuantity());
                } else if (type == 2) {
                    //出库库存相减
                    quantity = quantity.subtract(stock.getQuantity());
                    if (quantity.compareTo(BigDecimal.ZERO) < 0) {
                        throw new ServiceException("库存不足无法出库");
                    }
                } else {
                    throw new ServiceException("未知库存操作类型");
                }
                newStocks.setId(oldStocks.getId());
                newStocks.setQuantity(quantity);
            } else if (type == 2) {
                throw new ServiceException("以下商品库存不存在，无法出库：" + stock.getProductId());
            }

            data.add(newStocks);
            //创建出入库明细
            StockJournalDetails stockJournalDetails = new StockJournalDetailsDto();
            stockJournalDetails.setStockJournalId(stockJournalId);
            stockJournalDetails.setProductId(stock.getProductId());
            stockJournalDetails.setQuantity(stock.getQuantity());
            stockJournalDetailsList.add(stockJournalDetails);
        }
        //操作库存
        saveOrUpdateBatch(data);

        //返回库存操作明细列表
        return stockJournalDetailsList;
    }

    /**
     * 次品转良品
     */
    @Override
    public void defectiveToQualified(Stock stock) {
        if (ObjectUtil.isEmpty(stock.getId())) {
            throw new ServiceException("库存id不能为空");
        }
        if (ObjectUtil.isEmpty(stock.getQuantity())) {
            throw new ServiceException("要转换的数量不能为空");
        }
        Stock stock1 = getById(stock.getId());
        String victoriatouristJson = stock1.getVictoriatouristJson();
        JSONObject json = JSONObject.parseObject(victoriatouristJson);
        json.put("quantity", json.getBigDecimal("quantity").add(stock.getQuantity()));
        json.put("defectiveQuantity", json.getBigDecimal("defectiveQuantity").subtract(stock.getQuantity()));
        stock1.setVictoriatouristJson(json.toJSONString());
        updateById(stock1);
    }

    /**
     * 良品转次品
     */
    @Override
    public void qualifiedToDefective(Stock stock) {
        if (ObjectUtil.isEmpty(stock.getId())) {
            throw new ServiceException("库存id不能为空");
        }
        if (ObjectUtil.isEmpty(stock.getQuantity())) {
            throw new ServiceException("要转换的数量不能为空");
        }
        Stock stock1 = getById(stock.getId());
        String victoriatouristJson = stock1.getVictoriatouristJson();
        JSONObject json = JSONObject.parseObject(victoriatouristJson);
        json.put("quantity", json.getBigDecimal("quantity").subtract(stock.getQuantity()));
        json.put("defectiveQuantity", json.getBigDecimal("defectiveQuantity").add(stock.getQuantity()));
        stock1.setVictoriatouristJson(json.toJSONString());
        updateById(stock1);
    }


}
